#include "file_pager.h"
#include "mapped_file_object_server.h"

/* Initialise a pager for a file with a unique file identifier and shared page
   mapper for moderating access to loaded pages. */

FilePager::FilePager(fileid_t fileid, PageMapper *mapper, flags_t flags,
                     FilePaging *paging)
: Pager(mapper, flags), _paging(paging), fileid(fileid)
{
}

int FilePager::expected_items()
{
    return MappedFileObject_expected_items;
}

ipc_server_handler_type FilePager::handler()
{
    return (ipc_server_handler_type) handle_MappedFileObject;
}



/* Close the pager, removing the mapper from the paging registry if
   appropriate. */

void FilePager::close()
{
    _paging->detach_pager(fileid, _mapper);
}



/* File-specific operations. */

long FilePager::flush(offset_t populated_size, offset_t *size)
{
    return Pager::flush(populated_size, size);
}

long FilePager::resize(offset_t *size)
{
    return Pager::resize(size);
}

long FilePager::mmap(offset_t position, offset_t length, offset_t *start_pos,
                     offset_t *end_pos, offset_t *size)
{
    /* Set the limits of the paged region. */

    return Pager::mmap(position, length, start_pos, end_pos, size);
}



/* Generic pager operations. */

long FilePager::map(unsigned long offset, l4_addr_t hot_spot, flags_t flags, l4_snd_fpage_t *region)
{
    return Pager::map(offset, hot_spot, flags, region);
}

// vim: tabstop=4 expandtab shiftwidth=4
