#include <l4/re/c/dataspace.h>
#include <l4/re/consts.h>
#include <l4/sys/task.h>

#include "ipc.h"
#include "send_flexpage.h"



/* Make an L4 representation of the given flexpage. */

static l4_fpage_t ipc_get_fpage(SendFlexpage *send_flexpage)
{
    return l4_fpage(send_flexpage->base_addr, send_flexpage->order,
                    (send_flexpage->flags & L4RE_DS_MAP_FLAG_RW) ? L4_FPAGE_RW : L4_FPAGE_RO);
}

/* Make a representation of a flexpage for the IPC system. */

long ipc_prepare_flexpage(Flexpage *flexpage, offset_t offset,
                          offset_t max_offset, address_t hot_spot,
                          flags_t flags, l4_snd_fpage_t *region)
{
    SendFlexpage send_flexpage = flexpage->to_send(offset, hot_spot, flags,
                                                   max_offset);

    /* NOTE: Consider l4_fpage_invalid() as the fpage here. */

    if (!send_flexpage.order)
        return -L4_ERANGE;

    region->fpage = ipc_get_fpage(&send_flexpage);
    region->snd_base = hot_spot;

    return L4_EOK;
}

/* Unmap the given flexpage. */

void ipc_unmap_flexpage(Flexpage *flexpage)
{
    SendFlexpage send_flexpage = flexpage->to_unmap();

    l4_task_unmap(L4RE_THIS_TASK_CAP, ipc_get_fpage(&send_flexpage), L4_FP_OTHER_SPACES);
}

// vim: tabstop=4 expandtab shiftwidth=4
