#include <string.h>
#include <stdlib.h>

#include "region.h"



/* Initialise region state, indicating the size, file and position. */

RegionState::RegionState(unsigned long size, fileid_t fileid, offset_t filepos)
: size(size), fileid(fileid), filepos(filepos)
{
}

void RegionState::fill(fileid_t fileid, offset_t filepos)
{
    this->fileid = fileid;
    this->filepos = filepos;
}



/* Initialise a region having the given 'start' and 'end' addresses, with the
   'end' being one location beyond the last address in the region. */

Region::Region(address_t start, address_t end)
: start(start), end(end), state(end - start)
{
    /* Content state. */

    memset((void *) start, 0, end - start);
}

Region::~Region()
{
    free((void *) start);
}

length_t Region::size()
{
    return end - start;
}

/* Debugging methods. */

int Region::compare(Region *other)
{
    if (start < other->start)
        return -1;
    else if (start > other->start)
        return 1;
    else
        return 0;
}

void Region::fill(fileid_t fileid, offset_t filepos)
{
    state.fill(fileid, filepos);
}

void Region::flush()
{
}

/* Simulation methods. */

char *Region::read(length_t offset)
{
    if (offset < size())
        return (char *) start + offset;
    else
        return NULL;
}

void Region::write(const char *data, length_t offset)
{
    size_t length = strlen(data);

    if (offset + length < size())
        memcpy((void *) (start + offset), data, length + 1);
}

// vim: tabstop=4 expandtab shiftwidth=4
